module.exports = function(RED) {
  "use strict";

  var ffi = require('ffi');
  var path = require('path');

  var libnrbin = ffi.Library(path.join(__dirname, 'lib/libgpio'), {
    'led_usr_set': [ 'int', ['int'] ],
  });

  function SignalLEDNode(n) {
    // Create a RED node
    RED.nodes.createNode(this,n);

    var node = this;

    // Store local copies of the node configuration (as defined in the .html)
    this.led = n.led;
    this.initial = Number(n.initial);
    this.inverting = n.inverting;

    var inputCallback = function (msg) {
      var newState;

      if (msg.payload) {
        newState = true;
      } else {
        newState = false;
      }
      if (node.inverting) {
        newState = !newState;
      }

      libnrbin.led_usr_set(newState ? 1 : 0);
    };

    node.on("input", inputCallback);
    libnrbin.led_usr_set(this.initial);

    this.on("close", function() {
        // Called when the node is shutdown - eg on redeploy.
        // Allows ports to be closed, connections dropped etc.
        // eg: node.client.disconnect();
      });
    }

    // Register the node by name. This must be called before overriding any of the
    // Node functions.
    RED.nodes.registerType("signal_led", SignalLEDNode);

}
